/****************************************************************************
 *  File Name:          AT94K_INTERRUPTS.H
 *  Title:              FPSLIC (AT94K) MACROS
 *  Version:            1.1
 *  Last Updated:       May 12, 2000
 *  Target Device:      AT94K10/20/40
 *
 *  Author:             Taylor Cox
 *  Personal Email:     tcox@atmel.com
 *  Support Email:      fpslic@atmel.com
 *  Support Hotline:    +1 (408) 436-4119 (USA)
 *
 *  Revision:           1.1.1
 *
 *  Overview:           Provides MACRO definitions for enabling and disabling
 *                      the interrupt signals on the Atmel AT94K Device.  The
 *                      MACROS are used in the following manner:
 *
 *                      To ENABLE:  INTERRUPT_MACRO(ENABLE);
 *                      To DISABLE:  INTERRUPT_MACRO(DISABLE);
 *
 *
 *  Function Summary:   - GLOBAL_INT(x)
 *                          The global interrupt enable bit must be set (one)
 *                          for the interrupts to be enabled.  The individual
 *                          interrupt enable control is then performed in
 *                          seperate control registers.  If the global
 *                          interrupt is cleared (zero), none of the
 *                          interrupts are enabled independent of the
 *                          individual interrupt enable settings.
 *
 *                      - EXTERNAL_INTn(x) for {n | 0 .. 3, or S for all}
 *                          When the enable bit is set (one) and the global
 *                          interrupt is enabled, the corresponding External
 *                          pin interrupt is enabled.  The external
 *                          interrupts are always low-level-triggered.
 *
 *                      - FPGA_INT_ALL(x)
 *                          Enables\Disables all of the FPGA Interrupts with
 *                          one call.
 *
 *                      - FPGA_INT_MASKn(x) for {n | A .. D}
 *                          Enables\Disables the entire FPGA Interrupt Bank
 *                          with one call for each bank.
 *
 *                      - FPGA_INTn_MASK(x) for {n | 0 .. 15}
 *                          When a mask bit is set (one) and the global
 *                          interrupt is enabled, the given FPGA interrupt
 *                          is enabled.  The corresponding interrupt handling
 *                          vector is executed when the given FPGA interrupt
 *                          flag bit is set (one) by a low signal on the
 *                          associated interrupt line from the FPGA.
 *
 *                      - TIMER_INTS(x)
 *                          Enables/Disables all Timer interrupts with one call,
 *                          see individual interrupt calls for details.
 *
 *                      - TIMER1_INPUT_CAPTURE_INT(x)
 *                          When the Input Capture bit is set (one) and the
 *                          global interrupt is set (one), the Timer/Counter1
 *                          input capture event interrupt is enabled.  The
 *                          corresponding interrupt is executed if a capture-
 *                          trigggering event occurs.
 *
 *                      - TIMERn_OVERFLOW_INT(x) for {n | 0 .. 2}
 *                          When TIMERX_OVERFLOW_INT is set (one) and the
 *                          global interrupt is set (one), the Timer/CounterX
 *                          Overflow interrupt is enabled.  The corresponding
 *                          interrupt is executed if an overflow in
 *                          Timer/CounterX occurs.
 *
 *                      - TIMERn_OUTPUT_COMPARE_INT(x) for {n | 0 .. 2}
 *                          When the Output Compare bit is set (one) and the
 *                          global interrupt is set (one), the Timer/CounterX
 *                          Match interrupt is enabled.  The corresponding
 *                          interrupt is executed if a Compare match occurs.
 *
 *                      - UARTn_INTS(x) for {n| 0 .. 1}
 *                          When Enabled, all UARTn Interrupts are enabled, and
 *                          when disabled, all UARTn Interrupts are disabled.
 *
 *                      - UARTn_DATA_REG_EMPTY_INT(x) for {n | 0 .. 1}
 *                          When set (one), a setting of the UDREn bit in
 *                          USCRnA will cause the UART Data Register Empty
 *                          interrupt routine to be executed provided that the
 *                          global interrupts are enabled.
 *
 *                      - UARTn_RX_COMPLETE_INT(x) for {n | 0 .. 1}
 *                          When set (one), a setting of the RXCn bit in UCSRnA
 *                          will cause the Receive Complete interrupt routine
 *                          to be executed provided that global interrupts are
 *                          enabled.
 *
 *                      - UARTn_TX_COMPLETE_INT(x) for {n | 0 .. 1}
 *                          When set (one), a setting of the TXCn bit in UCSRnA
 *                          will cause the Transmit Complete interrupt routine
 *                          to be executed provided that global interrupts are
 *                          enabled.
 *
 *                      - SERIAL_INT(x)
 *                          When enabled (one), and the global interrupt is set,
 *                          the 2-Wire Serial Interrupt will be activated for as
 *                          long as the I2INT flag is HIGH.
 *
 *
 *
 *  Resources Used:     None
 *
 *  Notes:
 *      Revision 1.0.0 (TC)
 *          - Initial Release
 *          - Implemented Macros
 *      Revision 1.0.1 (TC)
 *          - Added Comments
 *          - Needs Simulation
 *      Revision 1.0.2 (TC)
 *          - Added Timer and UART Interrupts
 *          - Needs Simulation
 *      Revision 1.0.3 (TC)
 *          - Fixed Compile Time Errors
 *          - Simulated
 *      Revision 1.0.4 (ian)
 *          - changed uart interrupt register names to match
 *            data sheet change
 *      Revision 1.1.0 (TC)
 *          - Implemented Macros using the BIT Macro (Increased Readablity)
 *      Revision 1.1.1 (TC)
 *          - Added UART0 and UART1 Disable Macros
 *
 *  Known Issues:
 *      Revision 1.0.0
 *          - None at this time
 *      Revision 1.0.1
 *          - None at this time
 *      Revision 1.0.2
 *          - TIMER2_OUTPUT_COMPARE_INT(x) caused compile time ERROR
 *      Revision 1.0.3
 *          - None at this time
 *      Revision 1.1.0
 *          - FPGA_INTX_MASK don't work in IAR unless bits #defined in
 *            ioat94k.h
 ***************************************************************************/

#ifndef __at94k_interrupts_h
#define __at94k_interrupts_h

#include <ioat94k.h>
#include "at94k_utils.h"

/* Interrupt Constants */
#define DISABLE         0
#define ENABLE          1

/* Global Interrupt Enable */
#define GLOBAL_INT(x)                   ( (x) ? ( SREG |= 0x80 ) : ( SREG &= ~(0x80) ) )

/* External Interrupt Enables */
#define EXTERNAL_INTS(x)                ( (x) ? ( EIMF |= 0x0F      ) : ( EIMF &= ~(0x0F)      ) )
#define EXTERNAL_INT3(x)                ( (x) ? ( EIMF |= BIT(INT3) ) : ( EIMF &= ~(BIT(INT3)) ) )
#define EXTERNAL_INT2(x)                ( (x) ? ( EIMF |= BIT(INT2) ) : ( EIMF &= ~(BIT(INT2)) ) )
#define EXTERNAL_INT1(x)                ( (x) ? ( EIMF |= BIT(INT1) ) : ( EIMF &= ~(BIT(INT1)) ) )
#define EXTERNAL_INT0(x)                ( (x) ? ( EIMF |= BIT(INT0) ) : ( EIMF &= ~(BIT(INT0)) ) )

/* FPGA Interrupts */

#define FPGA_INT_ALL(x)                 ( (x) ? ( FISCR = 0x80, FISUD = 0x0F, FISUC = 0x0F, FISUB = 0x0F, FISUA = 0x0F ) : ( FISCR = 0x80, FISUD = 0x00, FISUC = 0x00, FISUB = 0x00, FISUA = 0x00 ) )
#define FPGA_INT_MASKD(x)               ( (x) ? ( FISCR = 0x80, FISUD |= 0x0F ) : ( FISCR = 0x80, FISUD &= ~( 0x0F ) ) )
#define FPGA_INT_MASKC(x)               ( (x) ? ( FISCR = 0x80, FISUC |= 0x0F ) : ( FISCR = 0x80, FISUC &= ~( 0x0F ) ) )
#define FPGA_INT_MASKB(x)               ( (x) ? ( FISCR = 0x80, FISUB |= 0x0F ) : ( FISCR = 0x80, FISUB &= ~( 0x0F ) ) )
#define FPGA_INT_MASKA(x)               ( (x) ? ( FISCR = 0x80, FISUA |= 0x0F ) : ( FISCR = 0x80, FISUA &= ~( 0x0F ) ) )

#define FPGA_INT15_MASK(x)              ( (x) ? ( FISCR = 0x80, FISUD |= BIT(FINT15) ) : ( FISCR = 0x80, FISUD &= ~(BIT(FINT15)) ) )
#define FPGA_INT14_MASK(x)              ( (x) ? ( FISCR = 0x80, FISUD |= BIT(FINT14) ) : ( FISCR = 0x80, FISUD &= ~(BIT(FINT14)) ) )
#define FPGA_INT13_MASK(x)              ( (x) ? ( FISCR = 0x80, FISUD |= BIT(FINT13) ) : ( FISCR = 0x80, FISUD &= ~(BIT(FINT13)) ) )
#define FPGA_INT12_MASK(x)              ( (x) ? ( FISCR = 0x80, FISUD |= BIT(FINT12) ) : ( FISCR = 0x80, FISUD &= ~(BIT(FINT12)) ) )
#define FPGA_INT11_MASK(x)              ( (x) ? ( FISCR = 0x80, FISUC |= BIT(FINT11) ) : ( FISCR = 0x80, FISUC &= ~(BIT(FINT11)) ) )
#define FPGA_INT10_MASK(x)              ( (x) ? ( FISCR = 0x80, FISUC |= BIT(FINT10) ) : ( FISCR = 0x80, FISUC &= ~(BIT(FINT10)) ) )
#define FPGA_INT9_MASK(x)               ( (x) ? ( FISCR = 0x80, FISUC |= BIT(FINT9)  ) : ( FISCR = 0x80, FISUC &= ~(BIT(FINT9))  ) )
#define FPGA_INT8_MASK(x)               ( (x) ? ( FISCR = 0x80, FISUC |= BIT(FINT8)  ) : ( FISCR = 0x80, FISUC &= ~(BIT(FINT8))  ) )
#define FPGA_INT7_MASK(x)               ( (x) ? ( FISCR = 0x80, FISUB |= BIT(FINT7)  ) : ( FISCR = 0x80, FISUB &= ~(BIT(FINT7))  ) )
#define FPGA_INT6_MASK(x)               ( (x) ? ( FISCR = 0x80, FISUB |= BIT(FINT6)  ) : ( FISCR = 0x80, FISUB &= ~(BIT(FINT6))  ) )
#define FPGA_INT5_MASK(x)               ( (x) ? ( FISCR = 0x80, FISUB |= BIT(FINT5)  ) : ( FISCR = 0x80, FISUB &= ~(BIT(FINT5))  ) )
#define FPGA_INT4_MASK(x)               ( (x) ? ( FISCR = 0x80, FISUB |= BIT(FINT4)  ) : ( FISCR = 0x80, FISUB &= ~(BIT(FINT4))  ) )
#define FPGA_INT3_MASK(x)               ( (x) ? ( FISCR = 0x80, FISUA |= BIT(FINT3)  ) : ( FISCR = 0x80, FISUA &= ~(BIT(FINT3))  ) )
#define FPGA_INT2_MASK(x)               ( (x) ? ( FISCR = 0x80, FISUA |= BIT(FINT2)  ) : ( FISCR = 0x80, FISUA &= ~(BIT(FINT2))  ) )
#define FPGA_INT1_MASK(x)               ( (x) ? ( FISCR = 0x80, FISUA |= BIT(FINT1)  ) : ( FISCR = 0x80, FISUA &= ~(BIT(FINT1))  ) )
#define FPGA_INT0_MASK(x)               ( (x) ? ( FISCR = 0x80, FISUA |= BIT(FINT0)  ) : ( FISCR = 0x80, FISUA &= ~(BIT(FINT0))  ) )

/* Timer Interrupt Enables */
#define TIMER_INTS(x)                   ( (x) ? ( TIMSK |= 0xFF        ) : ( TIMSK &= ~(0xFF)        ) )
#define TIMER1_INPUT_CAPTURE_INT(x)     ( (x) ? ( TIMSK |= BIT(TICIE1) ) : ( TIMSK &= ~(BIT(TICIE1)) ) )
#define TIMER2_OVERFLOW_INT(x)          ( (x) ? ( TIMSK |= BIT(TOIE2)  ) : ( TIMSK &= ~(BIT(TOIE2))  ) )
#define TIMER1_OVERFLOW_INT(x)          ( (x) ? ( TIMSK |= BIT(TOIE1)  ) : ( TIMSK &= ~(BIT(TOIE1))  ) )
#define TIMER0_OVERFLOW_INT(x)          ( (x) ? ( TIMSK |= BIT(TOIE0)  ) : ( TIMSK &= ~(BIT(TOIE0))  ) )
#define TIMER2_OUTPUT_COMPARE_INT(x)    ( (x) ? ( TIMSK |= BIT(OCIE2)  ) : ( TIMSK &= ~(BIT(OCIE2))  ) )
#define TIMER1_OUTPUT_COMPAREA_INT(x)   ( (x) ? ( TIMSK |= BIT(OCIE1A) ) : ( TIMSK &= ~(BIT(OCIE1A)) ) )
#define TIMER1_OUTPUT_COMPAREB_INT(x)   ( (x) ? ( TIMSK |= BIT(OCIE1B) ) : ( TIMSK &= ~(BIT(OCIE1B)) ) )
#define TIMER0_OUTPUT_COMPARE_INT(x)    ( (x) ? ( TIMSK |= BIT(OCIE0)  ) : ( TIMSK &= ~(BIT(OCIE0))  ) )

/* UART Interrupt Enables */
#define UART1_INTS(x)                   ( (x) ? ( UCSR1B |= (BIT(UDRIE1) | BIT(RXCIE1) | BIT(TXCIE1)) ) : ( UCSR1B &= ~(BIT(UDRIE1) | BIT(RXCIE1) | BIT(TXCIE1)) ) )
#define UART0_INTS(x)                   ( (x) ? ( UCSR0B |= (BIT(UDRIE0) | BIT(RXCIE0) | BIT(TXCIE0)) ) : ( UCSR0B &= ~(BIT(UDRIE0) | BIT(RXCIE0) | BIT(TXCIE0)) ) )

#define UART1_DATA_REG_EMPTY_INT(x)     ( (x) ? ( UCSR1B |= BIT(UDRIE1) ) : ( UCSR1B &= ~(BIT(UDRIE1)) ) )
#define UART0_DATA_REG_EMPTY_INT(x)     ( (x) ? ( UCSR0B |= BIT(UDRIE0) ) : ( UCSR0B &= ~(BIT(UDRIE0)) ) )
#define UART1_RX_COMPLETE_INT(x)        ( (x) ? ( UCSR1B |= BIT(RXCIE1) ) : ( UCSR1B &= ~(BIT(RXCIE1)) ) )
#define UART0_RX_COMPLETE_INT(x)        ( (x) ? ( UCSR0B |= BIT(RXCIE0) ) : ( UCSR0B &= ~(BIT(RXCIE0)) ) )
#define UART1_TX_COMPLETE_INT(x)        ( (x) ? ( UCSR1B |= BIT(TXCIE1) ) : ( UCSR1B &= ~(BIT(TXCIE1)) ) )
#define UART0_TX_COMPLETE_INT(x)        ( (x) ? ( UCSR0B |= BIT(TXCIE0) ) : ( UCSR0B &= ~(BIT(TXCIE0)) ) )

/* 2-Wire Serial Interface */
#define SERIAL_INT(x)                   ( (x) ? ( I2CR |= BIT(I2IE) ) : ( I2CR &= ~(BIT(I2IE)) ) )

#endif
